//
import { Component, html, css, raw } from 'wkit'
import { object2query, query2object } from '../utils.js'
import { __ROUTER_VIEW__, ROUTE_CALLBACKS } from '../constants.js'
import { watch } from '../store.js'

class RouterView extends Component {
  static props = {
    transition: false
  }

  static styles = css`
    :host {
      display: block;
    }
  `

  get current() {
    return this.#current
  }

  set current(v) {
    let old = this.#current
    this.#current = v
    if (this.keepAlive) {
      if (old) {
        if (this.$refs[old]) {
          this.$refs[old].removed = true
          this.$refs[old].deactivated()
          this.$refs[old].remove()
        } else {
          this.$requestUpdate()
        }
      } else {
        this.$requestUpdate()
      }
      if (v) {
        if (this.$refs[v]) {
          this.root.appendChild(this.$refs[v])
          this.$refs[v].$requestUpdate()
          if (this.transition) {
            this.$refs[v].$animate()
          }
          this.$refs[v].removed = false
          this.$refs[v].activated()
        } else {
          this.$requestUpdate()
        }
      }
    } else {
      this.$requestUpdate()
    }
  }

  #current = ''

  #views = []

  created() {
    window.wkitd.set(__ROUTER_VIEW__, this)
  }

  sync(views) {
    this.#views = views
  }

  render() {
    let option = {
      immediate: true,
      custom: [
        { transform: 'translateX(-32px)', opacity: 0 },
        { transform: 'translateX(0)', opacity: 1 }
      ]
    }

    if (this.current) {
      if (this.transition) {
        return raw(
          `<${this.current} ref="${this.current}" ${
            this.keepAlive ? 'keep-alive' : ''
          } #animation="%s"></${this.current}>`,
          [option]
        )
      }
      return raw(
        `<${this.current} ref="${this.current}" ${
          this.keepAlive ? 'keep-alive' : ''
        }></${this.current}>`
      )
    }
  }
}

class RouterLink extends Component {
  static props = {
    to: { type: null },
    disabled: false
  }

  static styles = css`
    :host {
      display: inline-flex;
      align-items: center;
      -webkit-user-select: none;
      user-select: none;
    }
    a {
      display: flex;
      align-items: center;
      justify-content: center;
      gap: var(--router-link-gap, 0);
      width: 100%;
      height: 100%;
      color: inherit;
      text-decoration: inherit;
      cursor: pointer;
    }

    :host([disabled]) a {
      opacity: 0.6;
      cursor: not-allowed;
    }
  `
  #to = { path: '' }
  #href = ''

  #navigate() {
    let type = this.$router.type

    if (this.disabled) {
      return
    }

    if (type === 'hash') {
      location.hash = this.#href
    } else {
      this.$router.push(this.#to)
    }
  }

  #parsePath() {
    let path, query, params
    if (typeof this.to === 'string') {
      let tmp = this.to.split('?')
      path = tmp[0]
      params = tmp[1] || ''
      query = query2object(params)
    } else {
      path = this.to.path || ''
      query = this.to.query || {}
      params =
        typeof query === 'string'
          ? query.replaceAll('?', '')
          : object2query(query)
    }

    path = '/' + path.replace(/^\/+/, '')
    this.#to = { path, query }

    if (params) {
      path += '?' + params
    }
    this.#href = path
  }

  activated() {
    this.mounted()
  }
  deactivated() {
    this.unmounted()
  }

  mounted() {
    watch('$route', route => {
      if (this.removed) {
        return
      }
      this.classList.toggle('active', route.path === this.#to.path)
    })
  }

  unmounted() {
    ROUTE_CALLBACKS.delete(this)
  }

  render() {
    this.#parsePath()
    return html`<a title=${this.#href} @click=${this.#navigate}
      ><slot></slot
    ></a>`
  }
}

RouterView.reg('view', 'router')
RouterLink.reg('link', 'router')
